//
// Copyright (c) 2013  Motoyuki Kasahara
//
// This program is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 2 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program.  If not, see <http://www.gnu.org/licenses/>.
//
package jp.sourceforge.gamerandomizer;

import java.io.File;
import java.util.ArrayList;
import java.util.Collections;
import java.util.Hashtable;
import java.util.List;
import java.util.Map;

import jp.sourceforge.gamerandomizerlib.Card;
import jp.sourceforge.gamerandomizerlib.CardComparator;
import jp.sourceforge.gamerandomizerlib.Expansion;
import jp.sourceforge.gamerandomizerlib.Game;
import jp.sourceforge.gamerandomizerlib.GameBuilder;
import jp.sourceforge.gamerandomizerlib.GameBuilderXMLFile;
import jp.sourceforge.gamerandomizerlib.Randomizer;
import android.app.Activity;
import android.app.AlertDialog;
import android.content.DialogInterface;
import android.os.Bundle;
import android.util.Log;
import android.view.Menu;
import android.view.MenuItem;
import android.view.View;
import android.widget.AdapterView;
import android.widget.AdapterView.OnItemClickListener;
import android.widget.Button;
import android.widget.ListView;
import android.widget.TextView;
import android.widget.Toast;

import jp.sourceforge.gamerandomizer.R;

public class CardListActivity extends Activity {
	// Path to a data directory.
	private String dataDirectoryPath;

	// Game.
	private Game game;

	// Randomizer.
	private Randomizer randomizer;

	// Cards selected by the randomizer.
	private List<Card> cards;

	// Check status of expansions of the current game.
	private List<Boolean> cardChecks;

	// An adapter for expansion listview.
	private CardListAdapter cardListAdapter;

	// Focus state (for save and restore state).
	enum FocusState {
		THIS_ACTIVITY, GAME_LIST_DIALOG, ERROR_DIALOG,
	};
	private FocusState focusState;

	// Saved title and message of error dialog
	// (for save and restore state).
	String lastErrorDialogTitle;
	String lastErrorDialogMessage;

	@Override
	protected void onCreate(Bundle savedInstanceState) {
		super.onCreate(savedInstanceState);
		setContentView(R.layout.activity_card_list);

		//
		// Initialize members.
		//
		dataDirectoryPath = null;
		game = null;
		randomizer = null;
		cards = new ArrayList<Card>();
		cardChecks = new ArrayList<Boolean>();
		cardListAdapter = new CardListAdapter(this);
		focusState = FocusState.THIS_ACTIVITY;
		lastErrorDialogTitle = "";
		lastErrorDialogMessage = "";

		//
		// Set a listener for the expansion list.
		//
		ListView cardListView = (ListView)
				findViewById(R.id.cardListView);

		cardListView.setOnItemClickListener(
				new OnItemClickListener() {
					@Override
					public void onItemClick(AdapterView<?> parent, 
							View view, int position, long ld) {
						ListView lv = (ListView)parent;
						cardChecks.set(position, lv.isItemChecked(position));
						cardListAdapter.notifyDataSetChanged();
						view.invalidate();
					}
				});

		//
		// Set initial data.
		//
		loadDataDirectoryPath();
		loadGame();
		drawActivity();
	}

	//
	// Handles an on-save-instance-state event of the activity.
	//
	@Override
	protected void onSaveInstanceState(Bundle state) {
		super.onSaveInstanceState(state);

		state.putString("focus-state", focusState.toString());
		int cardsSize = cards.size();
		for (int i = 0; i < cardsSize; i++)
			state.putString("card" + i, cards.get(i).getId());
		for (int i = 0; i < cardsSize; i++)
			state.putBoolean("card-check" + i, cardChecks.get(i));

		state.putString("error-dialog-title", lastErrorDialogTitle);
		state.putString("error-dialog-message", lastErrorDialogMessage);

		ListView lv = (ListView)findViewById(R.id.cardListView);
		state.putInt("listview-position", lv.getFirstVisiblePosition());
	}

	//
	// Handles an on-save-instance-state event of the activity.
	//
	@Override
	protected void onRestoreInstanceState(Bundle state) {
		super.onRestoreInstanceState(state);

		//
		// Read values saved in the state.
		//
		try {
			focusState = Enum.valueOf(FocusState.class,
					state.getString("focus-state"));
		} catch (Exception e) {
			focusState = FocusState.THIS_ACTIVITY;
		}

		int cardsSize = cards.size();
		for (int i = 0; i < cardsSize; i++) {
			String id = state.getString("card" + i);
			if (id != null)
				cards.set(i, game.getCard(id));
		}
		for (int i = 0; i < cardsSize; i++)
			cardChecks.set(i, state.getBoolean("card-check" + i));
		drawActivity();

		lastErrorDialogTitle = state.getString("error-dialog-title");
		if (lastErrorDialogTitle == null)
			lastErrorDialogTitle = "";
		lastErrorDialogMessage = state.getString("error-dialog-message");
		if (lastErrorDialogMessage == null)
			lastErrorDialogMessage = "";

		ListView lv = (ListView)findViewById(R.id.cardListView);
		lv.setSelection(state.getInt("listview-position"));

		//
		// Show a dialog if needed.
		//
		switch (focusState) {
		case ERROR_DIALOG:
			showErrorDialog(lastErrorDialogTitle,
					lastErrorDialogMessage);
			break;
		default:
			break;
		}
	}

	//
	// Handle an on-create event of the option menu.
	//
	@Override
	public boolean onCreateOptionsMenu(Menu menu) {
		// Inflate the menu; this adds items to the action bar if it is present.
		getMenuInflater().inflate(R.menu.card_list_option_menu, menu);
		return true;
	}

	//
	// Handle an on-options-item-selected event of the option menu.
	//
	@Override
	public boolean onOptionsItemSelected(MenuItem item) {
		switch (item.getItemId()) {
		case R.id.menuCheckAll2:
			checkAllCards();
			break;
		case R.id.menuUncheckAll2:
			uncheckAllCards();
		}
		return true;
	}

	//
	// Handle an on-click event of the "Select again!" button.
	//
	public void onDoReselectButtonClick(View view) {
		int reselectedCardsSize = reselectCards();
		if (reselectedCardsSize == 0) {
			Toast.makeText(this, R.string.no_card_is_checked,
					Toast.LENGTH_SHORT).show();
			return;
		}

		Map<String, Boolean> map = new Hashtable<String, Boolean>();
		int cardsSize = cards.size();
		for (int i = 0; i < cardsSize; i++)
			map.put(cards.get(i).getId(), cardChecks.get(i));
		Collections.sort(cards, new CardComparator());
		for (int i = 0; i < cardsSize; i++)
			cardChecks.set(i, map.get(cards.get(i).getId()));

		drawActivity();
	}

	//
	// Determine path to a data directory.
	//
	private void loadDataDirectoryPath() {
		File statusFileDir = getFilesDir();
		if (statusFileDir == null) {
			Log.e("setDataDirectoryPath", "getFilesDir() returns null");
			return;
		}
		DataDirectoryPathFileReader reader = null;
		try {
			File pathFile = new File(statusFileDir,
					getString(R.string.data_directory_path_file_name));
			reader = new DataDirectoryPathFileReader(pathFile);
			dataDirectoryPath = reader.getDataDirectoryPath();
		} catch (Exception e) {
			Log.e("setDataDirectoryPath", e.getMessage());
		}

		if (dataDirectoryPath == null) {
			showErrorDialog(getString(R.string.failed_to_load_game_data),
					getString(R.string.no_game_data_directory));			
		}
	}

	//
	// Load game data from XML files.
	//
	// The game data and status currently used are cleared.  If an error
	// occurs at loading data or no game is found, it shows an alert dialog.
	//
	private void loadGame() {
		//
		// Clear the current data.
		//
		game = null;
		randomizer = null;
		cards.clear();
		cardChecks.clear();
		if (dataDirectoryPath == null)
			return;

		//
		// Read a status file.
		//
		File statusFileDir = getFilesDir();
		if (statusFileDir == null)
			return;
		StatusFileReader reader = null;
		try {
			File statusFile = new File(statusFileDir,
					getString(R.string.status_file_name));
			reader = new StatusFileReader(statusFile);
		} catch (Exception e) {
			showErrorDialog(getString(R.string.failed_to_load_status_data),
					e.getMessage());
			return;
		}

		String lastGameId = reader.getLastGame();
		if (lastGameId == null) {
			return;
		}

		//
		// Load game data.
		//
		try {
			GameBuilder builder =
					new GameBuilderXMLFile(dataDirectoryPath);
			game = builder.build(lastGameId);
		} catch (Exception e) {
			Log.e("loadGame", e.getMessage(), e);
			showErrorDialog(getString(R.string.failed_to_load_game_data),
					e.getMessage());
			game = null;
			return;
		}
		if (game == null) {
			showErrorDialog(getString(R.string.failed_to_load_game_data),
					getString(R.string.no_game_data));
			return;
		}

		//
		// Set 'randomizer'.
		//
		int expansionsSize = game.getExpansionsSize();
		List<Expansion> xl = new ArrayList<Expansion>(expansionsSize);
		for (Expansion x : game.getExpansions()) {
			if (reader.getExpansionCheck(game.getId(), x.getId()))
				xl.add(x);
		}
		game.removeOtherExpansions(xl);

		//
		// Set 'cards' and 'cardChecks'.
		//
		randomizer = new Randomizer(game);
		randomizer.select(cards);
		int cardsSize = cards.size();
		for (int i = 0; i < cardsSize; i++)
			cardChecks.add(false);
		Collections.sort(cards, new CardComparator());
	}

	//
	// Draw a screen.
	// If the current game is selected, show its information.
	// Otherwise, show a screen which tells 'no game data'.
	//
	private void drawActivity() {
		if (randomizer == null)
			drawActivityNoGame();
		else
			drawActivityCurrentGame();
	}

	//
	// Draw a screen which tells 'no game data'.
	//
	private void drawActivityNoGame() {
		//
		// Hide the game title.  Show the "no game data" message instead.
		//
		TextView gameNameView = (TextView)findViewById(R.id.gameNameTextView2);
		gameNameView.setText(R.string.no_game_data);

		//
		// Clear the adapter.
		//
		cardListAdapter.clear();
		ListView cardListView = (ListView)findViewById(R.id.cardListView);
		cardListView.setAdapter(cardListAdapter);

		//
		// Disable the 'Select again!' button.
		//
		Button doReselectButton = (Button)findViewById(R.id.doReselectButton);
		doReselectButton.setEnabled(false);
	}

	//
	// Draw a screen which shows information about the current game.
	//
	private void drawActivityCurrentGame() {
		//
		// Show a title of the game.
		//
		TextView gameNameView = (TextView)findViewById(R.id.gameNameTextView2);
		gameNameView.setText(randomizer.getGame().getTitle());

		//
		// Show cards.
		//
		cardListAdapter.clear();
		for (Card c : cards) {
			CardListItem item = new CardListItem(
					c.getTitle(), c.getPrice(), c.getExpansion().getTitle());
			cardListAdapter.add(item);
		}
		ListView cardListView = (ListView)
				findViewById(R.id.cardListView);
		cardListView.setAdapter(cardListAdapter);

		//
		// Set checkboxes of the cards.
		//
		int cardsSize = cards.size();
		for (int i = 0; i < cardsSize; i++)
			cardListView.setItemChecked(i, cardChecks.get(i));

		//
		// Enable the 'Select again!' button.
		//
		Button doReselectButton = (Button)findViewById(R.id.doReselectButton);
		doReselectButton.setEnabled(true);

		if (cardsSize < game.getSelectionSize()) {
			Toast.makeText(this, R.string.not_enough_cards,
					Toast.LENGTH_LONG).show();
		}
	}

	//
	// Check all expansions of the current game.
	//
	private void checkAllCards() {
		ListView lv = (ListView)findViewById(R.id.cardListView);
		int cardsSize = cards.size();
		for (int i = 0; i < cardsSize; i++) {
			lv.setItemChecked(i, true);
			cardChecks.set(i, true);
		}
		lv.invalidate();
	}

	//
	// Uncheck all expansions of the current game.
	//
	private void uncheckAllCards() {
		ListView lv = (ListView)findViewById(R.id.cardListView);
		int cardsSize = cards.size();
		for (int i = 0; i < cardsSize; i++) {
			lv.setItemChecked(i, false);
			cardChecks.set(i, false);
		}
		lv.invalidate();
	}

	//
	// Select cards again.
	//
	private int reselectCards() {
		List<Card> cl = new ArrayList<Card>();
		List<Boolean> bl = new ArrayList<Boolean>();

		int cardsSize = cards.size();
		for (int i = 0; i < cardsSize; i++) {
			if (!cardChecks.get(i)) {
				cl.add(cards.get(i));
				bl.add(false);
			}
		}

		int reselectedCardsSize = cardsSize - cl.size();
		cards = cl;
		cardChecks = bl;
		randomizer.select(cards);
		for (int i = cardChecks.size(); i < cardsSize; i++)
			cardChecks.add(true);

		return reselectedCardsSize;
	}

	//
	// Show an error dialog.
	//
	private void showErrorDialog(String title, String message) {
		lastErrorDialogTitle = title;
		lastErrorDialogMessage = message;

		AlertDialog.Builder dialog = new AlertDialog.Builder(this);
		dialog.setTitle(title);
		dialog.setMessage(message);
		dialog.setPositiveButton(android.R.string.ok,
				new DialogInterface.OnClickListener() {
			@Override
			public void onClick(DialogInterface iface, int which) {
				focusState = FocusState.THIS_ACTIVITY;
			}
		});
		dialog.setOnCancelListener(
				new DialogInterface.OnCancelListener() {
			@Override
			public void onCancel(DialogInterface dialog) {
				focusState = FocusState.THIS_ACTIVITY;
			}
		});
		focusState = FocusState.ERROR_DIALOG;
		dialog.show();
	}
}
