//
// Copyright (c) 2013  Motoyuki Kasahara
//
// This program is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 2 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program.  If not, see <http://www.gnu.org/licenses/>.
//
package jp.sourceforge.gamerandomizer;

import java.io.File;
import java.util.ArrayList;
import java.util.Collections;
import java.util.Comparator;
import java.util.Hashtable;
import java.util.List;

import jp.sourceforge.gamerandomizerlib.Expansion;
import jp.sourceforge.gamerandomizerlib.Game;
import jp.sourceforge.gamerandomizerlib.GameBuilder;
import jp.sourceforge.gamerandomizerlib.GameBuilderXMLFile;
import android.app.Activity;
import android.app.AlertDialog;
import android.content.DialogInterface;
import android.content.Intent;
import android.os.Bundle;
import android.util.Log;
import android.view.Menu;
import android.view.MenuItem;
import android.view.View;
import android.widget.AdapterView;
import android.widget.AdapterView.OnItemClickListener;
import android.widget.Button;
import android.widget.ListView;
import android.widget.TextView;
import android.widget.Toast;

public class MainActivity extends Activity {
	// Path to a data directory.
	private String dataDirectoryPath;

	// Loaded games.
	private List<Game> games;

	// Check status of expansions of the current game.
	private Hashtable<Game, List<Boolean>> expansionChecks;

	// A game currently shown on the screen.
	private Game currentGame;

	// A game the user selects.  (used in the game-selection dialog)
	private Game nextGame;

	// The number of candidate cards.
	private int cardsSize;

	// An adapter for expansion listview.
	private ExpansionListAdapter expansionListAdapter;

	// Focus state (for save and restore state).
	enum FocusState {
		THIS_ACTIVITY, GAME_LIST_DIALOG, ERROR_DIALOG,
	};
	private FocusState focusState;

	// Saved title and message of error dialog
	// (for save and restore state).
	String lastErrorDialogTitle;
	String lastErrorDialogMessage;

	// Request code for "MoreActivity".
	private static final int REQUEST_CODE_MORE = 1;

	//
	// Handle an on-create event of the activity.
	//
	@Override
	protected void onCreate(Bundle state) {
		super.onCreate(state);
		setContentView(R.layout.activity_main);

		//
		// Initialize members.
		//
		dataDirectoryPath = null;
		games = new ArrayList<Game>();
		expansionChecks = new Hashtable<Game, List<Boolean>>();
		currentGame = null;
		nextGame = null;
		cardsSize = 0;
		expansionListAdapter = new ExpansionListAdapter(this);
		focusState = FocusState.THIS_ACTIVITY;
		lastErrorDialogTitle = "";
		lastErrorDialogMessage = "";
		
		//
		// Set initial data.
		//
		loadDataDirectoryPath();
		loadGames();
		loadStatus();

		//
		// Set a listener for the expansion list.
		//
		ListView expansionListView = (ListView)
				findViewById(R.id.expansionListView);
		expansionListView.setOnItemClickListener(
				new OnItemClickListener() {
					@Override
					public void onItemClick(AdapterView<?> parent, 
							View view, int position, long ld) {
						ListView lv = (ListView)parent;
						expansionChecks.get(currentGame).set(position, 
								lv.isItemChecked(position));
						expansionListAdapter.notifyDataSetChanged();
						view.invalidate();
						postProcExpansionCheckUpdated();
					}
				});

		//
		// Draw the activity.
		//
		drawActivity();
	}

	//
	// Handle an on-stop event of the activity.
	//
	@Override
	protected void onStop() {
		super.onStop();
		saveStatus();
	}

	//
	// Handles an on-save-instance-state event of the activity.
	//
	@Override
	protected void onSaveInstanceState(Bundle state) {
		super.onSaveInstanceState(state);

		saveStatus();
		state.putString("focus-state", focusState.toString());
		if (nextGame != null)
			state.putString("next-game", nextGame.getId());

		state.putString("error-dialog-title", lastErrorDialogTitle);
		state.putString("error-dialog-message", lastErrorDialogMessage);

		if (currentGame != null) {
			ListView lv = (ListView)findViewById(R.id.expansionListView);
			Expansion x = currentGame.getExpansion(
					lv.getFirstVisiblePosition());
			state.putString("listview-position", x.getId());
		}
	}

	//
	// Handles an on-restore-instance-state event of the activity.
	//
	@Override
	protected void onRestoreInstanceState(Bundle state) {
		super.onRestoreInstanceState(state);

		//
		// Read values saved in the state.
		//
		try {
			focusState = Enum.valueOf(FocusState.class,
					state.getString("focus-state"));
		} catch (Exception e) {
			focusState = FocusState.THIS_ACTIVITY;
		}

		String id = state.getString("next-game");
		int index = -1;
		if (id != null)
			index = games.indexOf(new Game(id, "", 0));
		if (index >= 0)
			nextGame = games.get(index);

		lastErrorDialogTitle = state.getString("error-dialog-title");
		if (lastErrorDialogTitle == null)
			lastErrorDialogTitle = "";
		lastErrorDialogMessage = state.getString("error-dialog-message");
		if (lastErrorDialogMessage == null)
			lastErrorDialogMessage = "";

		ListView lv = (ListView)findViewById(R.id.expansionListView);
		String lvPosition = state.getString("listview-position");
		if (lvPosition != null) {
			int gamesSize = games.size();
			for (int i = 0; i < gamesSize; i++) {
				if (games.get(i).getId().equals(lvPosition)) {
					lv.setSelection(i);
					break;
				}
			}
		}

		//
		// Show a dialog if needed.
		//
		switch (focusState) {
		case GAME_LIST_DIALOG:
			switchGame(nextGame);
			break;
		case ERROR_DIALOG:
			showErrorDialog(lastErrorDialogTitle,
					lastErrorDialogMessage);
			break;
		default:
			break;
		}
	}

	//
	// Handle an on-create event of the option menu.
	//
	@Override
	public boolean onCreateOptionsMenu(Menu menu) {
		// Inflate the menu; this adds items to the action bar if it is present.
		getMenuInflater().inflate(R.menu.main_option_menu, menu);
		return true;
	}

	//
	// Handle an on-options-item-selected event of the option menu.
	//
	@Override
	public boolean onOptionsItemSelected(MenuItem item) {
		switch (item.getItemId()) {
		case R.id.menuSwitchGame:
			switchGame();
			break;
		case R.id.menuCheckAll:
			checkAllExpansions();
			break;
		case R.id.menuUncheckAll:
			uncheckAllExpansions();
			break;
		case R.id.menuMore:
			saveStatus();
			Intent in = new Intent(this, MoreActivity.class);
			startActivityForResult(in, REQUEST_CODE_MORE);
			break;
		}
		return true;
	}

	//
	// Get result of an activity.
	//
	@Override
	protected void onActivityResult(int requestCode, int resultCode,
			Intent data) {
		if (resultCode != RESULT_OK)
			return;
		if (requestCode == REQUEST_CODE_MORE) {
			loadDataDirectoryPath();
			loadGames();
			loadStatus();
			drawActivity();
		}
	}

	//
	// Handle an on-click event of the "Select!" button.
	//
	public void onDoSelectButtonClick(View view) {
		if (cardsSize < currentGame.getSelectionSize()) {
			Toast.makeText(this, R.string.not_enough_cards,
					Toast.LENGTH_SHORT).show();
			return;
		}

		saveStatus();
		Intent in = new Intent(this, CardListActivity.class);
		startActivity(in);
	}

	//
	// Determine path to a data directory.
	//
	private void loadDataDirectoryPath() {
		File statusFileDir = getFilesDir();
		if (statusFileDir == null) {
			Log.e("setDataDirectoryPath", "getFilesDir() returns null");
			return;
		}
		DataDirectoryPathFileReader reader = null;
		try {
			File pathFile = new File(statusFileDir,
					getString(R.string.data_directory_path_file_name));
			reader = new DataDirectoryPathFileReader(pathFile);
			dataDirectoryPath = reader.getDataDirectoryPath();
		} catch (Exception e) {
			Log.e("setDataDirectoryPath", e.getMessage());
		}

		if (dataDirectoryPath == null) {
			showErrorDialog(getString(R.string.failed_to_load_game_data),
					getString(R.string.no_game_data_directory));			
		}
	}

	//
	// Load game data from XML files.
	//
	// The game data and status currently used are cleared.  If an error
	// occurs at loading data or no game is found, it shows an alert dialog.
	//
	private void loadGames() {
		//
		// Clear the current data.
		//
		games.clear();
		expansionChecks.clear();
		currentGame = null;

		if (dataDirectoryPath == null)
			return;

		//
		// Load game data.
		//
		try {
			GameBuilder builder =
					new GameBuilderXMLFile(dataDirectoryPath);
			builder.buildAll(games);
		} catch (Exception e) {
			Log.e("loadGames", e.getMessage(), e);
			showErrorDialog(getString(R.string.failed_to_load_game_data),
					e.getMessage());
			games.clear();
			return;
		}

		int gamesSize = games.size();
		if (gamesSize == 0) {
			showErrorDialog(getString(R.string.no_game_data), dataDirectoryPath);
			return;
		}
		Collections.sort(games, new Comparator<Game>() {
			@Override
			public int compare(Game g1, Game g2) {
				return g1.getTitle().compareToIgnoreCase(g2.getTitle());
			}
		});
		
		//
		// Set 'expansionChecks'.
		//
		for (Game g : games) {
			int expansionsSize = g.getExpansionsSize();
			List<Boolean> bl = new ArrayList<Boolean>(expansionsSize);
			for (int j = 0; j < expansionsSize; j++)
				bl.add(false);
			expansionChecks.put(g, bl);
		}
	}

	//
	// Load a status file.
	//
	private void loadStatus() {
		if (games.size() == 0)
			return;

		//
		// Open the status file.
		//
		File statusFileDir = getFilesDir();
		if (statusFileDir == null) {
			Log.e("saveStatus", "getFilesDir() returns null");
			return;
		}
		StatusFileReader reader = null;
		try {
			File statusFile = new File(statusFileDir,
					getString(R.string.status_file_name));
			reader = new StatusFileReader(statusFile);
		} catch (Exception e) {
			Log.e("loadStatus", e.getMessage());
			return;
		}

		//
		// Set 'currentGame'.
		//
		currentGame = games.get(0);
		String lastGameName = reader.getLastGame();
		if (lastGameName != null) {
			int gameIndex = games.indexOf(new Game(lastGameName, "", 0));
			if (gameIndex >= 0)
				currentGame = games.get(gameIndex);
		}

		//
		// Set 'expansionChecks'.
		//
		for (Game g : games) {
			List<Boolean> bl = expansionChecks.get(g);
			int blSize = bl.size();
			for (int i = 0; i < blSize; i++) {
				bl.set(i, reader.getExpansionCheck(g.getId(),
						g.getExpansion(i).getId()));
			}
			if (bl.indexOf(true) < 0)
				bl.set(0, true);
		}
	}

	//
	// Save a status file.
	//
	private void saveStatus() {
		//
		// Open the status file.
		//
		File statusFileDir = getFilesDir();
		if (statusFileDir == null) {
			Log.e("saveStatus", "getFilesDir() returns null");
			return;
		}
		File statusFile = new File(statusFileDir,
				getString(R.string.status_file_name));
		StatusFileWriter writer = null;
		try {
			writer = new StatusFileWriter(statusFile);
		} catch (Exception e) {
			Log.e("saveStatus", e.getMessage());
			return;			
		}

		//
		// Put 'currentGame'.
		//
		if (currentGame != null)
			writer.setLastGame(currentGame.getId());

		//
		// Put 'expansionChecks'.
		//
		for (Game g : games) {
			List<Boolean> bl = expansionChecks.get(g);
			int blSize = bl.size();
			for (int i = 0; i < blSize; i++) {
				writer.setExpansionCheck(g.getId(), 
						g.getExpansion(i).getId(), bl.get(i));
			}
		}

		try {
			writer.writeFile();
		} catch (Exception e) {
			Log.e("saveStatus", e.getMessage());
		}
	}

	//
	// Draw a screen.
	// If the current game is selected, show its information.
	// Otherwise, show a screen which tells 'no game data'.
	//
	private void drawActivity() {
		if (currentGame == null)
			drawActivityNoGame();
		else
			drawActivityCurrentGame();
	}

	//
	// Draw a screen which tells 'no game data'.
	//
	private void drawActivityNoGame() {
		//
		// Hide the game title.  Show the "no game data" message instead.
		//
		TextView gameNameView = (TextView)findViewById(R.id.gameNameTextView);
		gameNameView.setText(R.string.no_game_data);

		//
		// Clear the adapter.
		//
		expansionListAdapter.clear();
		ListView expansionView = (ListView)
				findViewById(R.id.expansionListView);
		expansionView.setAdapter(expansionListAdapter);

		//
		// Hide the total number of cards used for random selection.
		//
		TextView cardsSizeView = (TextView)findViewById(R.id.totalCardsSizeTextView);
		cardsSizeView.setText("");

		//
		// Disable the 'Select!' button.
		//
		Button doSelectButton = (Button)findViewById(R.id.doSelectButton);
		doSelectButton.setEnabled(false);
	}

	//
	// Draw a screen which shows information about the current game.
	//
	private void drawActivityCurrentGame() {
		//
		// Show a title of the game.
		//
		TextView gameNameView = (TextView)findViewById(R.id.gameNameTextView);
		gameNameView.setText(currentGame.getTitle());

		//
		// Show expansions.
		//
		expansionListAdapter.clear();
		List<Expansion> expansions = currentGame.getExpansions();
		for (Expansion x : expansions) {
			ExpansionListItem item = new ExpansionListItem(
					String.format(getString(R.string.use_expansion),
							x.getTitle()), currentGame.getCardsSize(x));
			expansionListAdapter.add(item);			
		}
		ListView expansionListView = (ListView)
				findViewById(R.id.expansionListView);
		expansionListView.setAdapter(expansionListAdapter);

		//
		// Set checkboxes of the expansions.
		//
		int expansionsSize = expansions.size();
		for (int i = 0; i < expansionsSize; i++) {
			expansionListView.setItemChecked(i, 
					expansionChecks.get(currentGame).get(i));
		}

		//
		// Enable the 'Select!' button.
		//
		Button doSelectButton = (Button)findViewById(R.id.doSelectButton);
		doSelectButton.setEnabled(true);

		//
		// Show the total number of cards.
		//
		postProcExpansionCheckUpdated();
	}

	//
	// Show the total number of cards.
	// Also enable/disable the "Select!" button.
	//
	private void postProcExpansionCheckUpdated() {
		if (currentGame == null)
			return;

		Game g = new Game(currentGame);
		List<Boolean> bl = expansionChecks.get(currentGame);
		for (int i = 0; i < bl.size(); i++) {
			if (!bl.get(i)) {
				Expansion x = currentGame.getExpansion(i);
				g.removeExpansion(x);
			}
		}
		cardsSize = g.getCardsSize();

		TextView tv = (TextView)findViewById(R.id.totalCardsSizeTextView);
		tv.setText(String.format(getString(R.string.total_n_cards),
				cardsSize, currentGame.getCardsSize()));
	}

	//
	// Show a game selection dialog and switch the current game.
	//
	public void switchGame() {
		switchGame(currentGame);
	}

	public void switchGame(Game checkedGame) {
		if (games.size() == 0)
			return;

		String[] items = new String[games.size()];
		int i = 0;
		for (Game g : games)
			items[i++] = g.getTitle();

		int checkedIndex;
		if (checkedGame != null)
			nextGame = checkedGame;
		else if (currentGame != null)
			nextGame = currentGame;
		else
			nextGame = games.get(0);
		checkedIndex = games.indexOf(nextGame);

		AlertDialog.Builder dialog = new AlertDialog.Builder(this);
		dialog.setTitle(R.string.switch_a_game);
		dialog.setSingleChoiceItems(items, checkedIndex,
				new DialogInterface.OnClickListener() {
			@Override
			public void onClick(DialogInterface iface, int which) {
				nextGame = games.get(which);
			}
		});
		dialog.setPositiveButton(android.R.string.ok,
				new DialogInterface.OnClickListener() {
			@Override
			public void onClick(DialogInterface ifece, int which) {
				if (currentGame != nextGame) {
					currentGame = nextGame;
					drawActivity();
				}
				nextGame = null;
				focusState = FocusState.THIS_ACTIVITY;
			}
		});
		dialog.setOnCancelListener(
				new DialogInterface.OnCancelListener() {
			@Override
			public void onCancel(DialogInterface dialog) {
				nextGame = null;
				focusState = FocusState.THIS_ACTIVITY;
			}
		});
		focusState = FocusState.GAME_LIST_DIALOG;
		dialog.show();
	}

	//
	// Check all expansions of the current game.
	//
	private void checkAllExpansions() {
		ListView lv = (ListView)findViewById(R.id.expansionListView);
		int expansionsSize = currentGame.getExpansionsSize();
		for (int i = 0; i < expansionsSize; i++) {
			lv.setItemChecked(i, true);
			expansionChecks.get(currentGame).set(i, true);
		}
		lv.invalidate();
		postProcExpansionCheckUpdated();
	}

	//
	// Uncheck all expansions of the current game.
	//
	private void uncheckAllExpansions() {
		ListView lv = (ListView)findViewById(R.id.expansionListView);
		int expansionsSize = currentGame.getExpansionsSize();
		for (int i = 0; i < expansionsSize; i++) {
			lv.setItemChecked(i, false);
			expansionChecks.get(currentGame).set(i, false);
		}
		lv.invalidate();
		postProcExpansionCheckUpdated();
	}

	//
	// Show an error dialog.
	//
	private void showErrorDialog(String title, String message) {
		lastErrorDialogTitle = title;
		lastErrorDialogMessage = message;

		AlertDialog.Builder dialog = new AlertDialog.Builder(this);
		dialog.setTitle(title);
		dialog.setMessage(message);
		dialog.setPositiveButton(android.R.string.ok,
				new DialogInterface.OnClickListener() {
			@Override
			public void onClick(DialogInterface iface, int which) {
				focusState = FocusState.THIS_ACTIVITY;
			}
		});
		dialog.setOnCancelListener(
				new DialogInterface.OnCancelListener() {
			@Override
			public void onCancel(DialogInterface dialog) {
				focusState = FocusState.THIS_ACTIVITY;
			}
		});
		focusState = FocusState.ERROR_DIALOG;
		dialog.show();
	}
}
