//
// Copyright (c) 2013  Motoyuki Kasahara
//
// This program is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 2 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program.  If not, see <http://www.gnu.org/licenses/>.
//
package jp.sourceforge.gamerandomizer;

import java.io.File;
import java.util.ArrayList;
import java.util.List;

import android.app.AlertDialog;
import android.app.ListActivity;
import android.content.DialogInterface;
import android.content.Intent;
import android.content.pm.PackageInfo;
import android.content.pm.PackageManager;
import android.content.pm.PackageManager.NameNotFoundException;
import android.net.Uri;
import android.os.Bundle;
import android.util.Log;
import android.view.View;
import android.widget.ArrayAdapter;
import android.widget.ListView;
import android.widget.Toast;

public class MoreActivity extends ListActivity {
	// Adapter of the list.
	private ArrayAdapter<String> adapter;

	// List items.
	private static final int[] listItems = {
		R.string.reload_data,
		R.string.select_data_folder,
		R.string.about_this_application,
		R.string.visit_official_web_site,
	};

	// Reloading data files is requested or not.
	boolean reloadDataFlag;

	// Focus place.
	enum FocusState {
		THIS_ACTIVITY, ABOUT_THIS_APPLICATION_DIALOG,
	};
	private FocusState focusState;

	// Request code for "DirectorySelectionActivity".
	private static final int REQUEST_CODE_DIRECTORY_SELECTION = 1;

	//
	// Handle an on-create event of the activity.
	//
	@Override
	protected void onCreate(Bundle savedInstanceState) {
		super.onCreate(savedInstanceState);

		//
		// Setup the listview.
		//
		List<String> titles = new ArrayList<String>();
		for (int i = 0; i < listItems.length; i++)
			titles.add(getString(listItems[i]));

		adapter = new ArrayAdapter<String>(this,
				android.R.layout.simple_list_item_1, titles);
		this.setListAdapter(adapter);

		//
		// Initialize members.
		//
		reloadDataFlag = false;
		focusState = FocusState.THIS_ACTIVITY;
	}

	//
	// Handles an on-save-instance-state event of the activity.
	//
	@Override
	protected void onSaveInstanceState(Bundle state) {
		super.onSaveInstanceState(state);
		state.putString("focus-state", focusState.toString());

		state.putInt("listview-position",
				getListView().getFirstVisiblePosition());
	}

	//
	// Handles an on-restore-instance-state event of the activity.
	//
	@Override
	protected void onRestoreInstanceState(Bundle state) {
		super.onRestoreInstanceState(state);

		//
		// Read values saved in the state.
		//
		try {
			focusState = Enum.valueOf(FocusState.class,
					state.getString("focus-state"));
		} catch (Exception e) {
			focusState = FocusState.THIS_ACTIVITY;
		}

		getListView().setSelection(state.getInt("listview-position"));

		//
		// Show a dialog if needed.
		//
		switch (focusState) {
		case ABOUT_THIS_APPLICATION_DIALOG:
			showAboutApplicationDialog();
			break;
		default:
			break;
		}
	}

	//
	// Handle an on-click event of the list.
	//
	@Override
	protected void onListItemClick(ListView listView, View view,
			int position, long id) {
		super.onListItemClick(listView, view, position, id);

		if (position < 0 || position >= listItems.length)
			return;

		switch (listItems[position]) {
		case R.string.reload_data:
			reloadDataFlag = true;
			setResult();
			Toast.makeText(this, R.string.game_data_will_be_reloaded,
					Toast.LENGTH_LONG).show();
			break;
		case R.string.select_data_folder:
			Intent in = new Intent(this,
					DirectorySelectionActivity.class);
			startActivityForResult(in, REQUEST_CODE_DIRECTORY_SELECTION);
			break;
		case R.string.about_this_application:
			showAboutApplicationDialog();
			break;
		case R.string.visit_official_web_site:
			Intent in2 = new Intent(Intent.ACTION_VIEW,
					Uri.parse(getString(R.string.app_uri)));
			startActivity(in2);
			break;
		}
	}

	//
	// Get result of an activity.
	//
	@Override
	protected void onActivityResult(int requestCode, int resultCode,
			Intent data) {
		if (resultCode != RESULT_OK)
			return;
		if (requestCode == REQUEST_CODE_DIRECTORY_SELECTION) {
			String path = data.getStringExtra("path");
			if (path != null) {
				saveDataDirectoryPath(path);
				reloadDataFlag = true;
				setResult();
			}
		}
	}

	//
	// Set result of this activity.
	//
	protected void setResult() {
		Intent in = new Intent();
		in.putExtra("reload", reloadDataFlag);
		setResult(RESULT_OK, in);
	}

	//
	// Save a data directory path file.
	//
	private void saveDataDirectoryPath(String path) {
		//
		// Open the status file.
		//
		File statusFileDir = getFilesDir();
		if (statusFileDir == null) {
			Log.e("saveDataDirectoryPath", "getFilesDir() returns null");
			return;
		}
		File pathFile = new File(statusFileDir,
				getString(R.string.data_directory_path_file_name));
		DataDirectoryPathFileWriter writer = null;
		try {
			writer = new DataDirectoryPathFileWriter(pathFile);
			writer.writeFile(path);
		} catch (Exception e) {
			Log.e("saveDataDirectoryPath", e.getMessage());
			return;			
		}
	}

	//
	// Show the "About this application" dialog.
	//
	private void showAboutApplicationDialog() {
		PackageManager manager = this.getPackageManager();
		String version = "";
		String copyright = getString(R.string.copyright);

		try {
			PackageInfo info = manager.getPackageInfo(
					this.getPackageName(), PackageManager.GET_ACTIVITIES);
			version = String.format(getString(R.string.version),
					info.versionName);
		} catch (NameNotFoundException e) {
			; // Nothing to do.
		}

		AlertDialog.Builder dialog = new AlertDialog.Builder(this);
		dialog.setTitle(R.string.about_this_application);
		dialog.setMessage(version + "\n" + copyright);
		dialog.setIcon(R.drawable.ic_launcher);
		dialog.setPositiveButton(android.R.string.ok,
				new DialogInterface.OnClickListener() {
			@Override
			public void onClick(DialogInterface iface, int which) {
				focusState = FocusState.THIS_ACTIVITY;
			}
		});
		dialog.setOnCancelListener(
				new DialogInterface.OnCancelListener() {
			@Override
			public void onCancel(DialogInterface dialog) {
				focusState = FocusState.THIS_ACTIVITY;
			}
		});
		focusState = FocusState.ABOUT_THIS_APPLICATION_DIALOG;
		dialog.show();
	}
}
