//
// Copyright (c) 2013  Motoyuki Kasahara
//
// This program is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 2 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.	 See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program.	 If not, see <http://www.gnu.org/licenses/>.
//
package jp.sourceforge.gamerandomizerlib;

import java.io.Serializable;
import java.util.Comparator;

public class CardComparator implements Comparator<Card>,
									   Serializable {
	private CardComparisonMode mode;

	/**
	 * Constructs an object.
	 *
	 * Comparison mode of the constructed object is set to
	 * 'PRICE_THEN_EXPANSION'.
	 */
	public CardComparator() {
		mode = CardComparisonMode.PRICE_THEN_EXPANSION;
	}

	/**
	 * Constructs an object.
	 *
	 * Comparison mode of the constructed object is set to the specified
	 * value.
	 *
	 * @param modeArg  Comparison mode
	 */
	public CardComparator(CardComparisonMode modeArg) {
		assert (mode == CardComparisonMode.PRICE_THEN_EXPANSION
			|| mode == CardComparisonMode.EXPANSION_THEN_PRICE);
		mode = modeArg;
	}

	/**
	 * Returns comparision mode.
	 *
	 * @return  comparision mode
	 */
	public CardComparisonMode getMode() {
		return mode;
	}

	/**
	 * Compares the specified two cards.
	 *
	 * It compares two card objects 'c1' and 'c2'.  It returns an integer
	 * less than, equal to, or greater than zero if 'c1' is respectively
	 * less than, equal to, or greater than 'c2'.
	 *
	 * @param c1  a card
	 * @param c2  a card
	 * @return  a negative integer, zero, or a positive integer according
	 *          with result of the comparison.
	 */
	@Override
	public int compare(Card c1, Card c2) {
		int result = 0;

		switch (mode) {
		case PRICE_THEN_EXPANSION:
			result = comparePrice(c1, c2);
			if (result == 0)
				result = compareExpansion(c1, c2);
			break;
		case EXPANSION_THEN_PRICE:
			result = compareExpansion(c1, c2);
			if (result == 0)
				result = comparePrice(c1, c2);
			break;
		}

		return result;
	}

	//
	// Internal method of compare().
	//
	// It compares two cards with their price values.
	//
	private int comparePrice(Card c1, Card c2) {
		String price1 = c1.getPrice();
		int length1 = price1.length();

		String price2 = c2.getPrice();
		int length2 = price2.length();

		int idx1 = 0;
		int idx2 = 0;

		for (;;) {
			if (idx1 >= length1 || idx2 >= length2)
				return length1 - length2;
			int p1 = price1.codePointAt(idx1);
			int p2 = price2.codePointAt(idx2);
			if ('0' <= p1 && p1 <= '9' && '0' <= p2 && p2 <= '9') {
				int num1 = p1 - '0';
				int num2 = p2 - '0';
				idx1++;
				idx2++;
				for (;;) {
					boolean updated = false;
					if (idx1 < length1) {
						p1 = price1.codePointAt(idx1);
						if ('0' <= p1 && p1 <= '9') {
							num1 = num1 * 10 + (p1 - '0');
							updated = true;
							idx1++;
						}
					}
					if (idx2 < length2) {
						p2 = price2.codePointAt(idx2);
						if ('0' <= p2 && p2 <= '9') {
							num2 = num2 * 10 + (p2 - '0');
							updated = true;
							idx2++;
						}
					}
					if (!updated)
						break;
				}
				if (num1 != num2)
					return num1 - num2;
			} else {
				if (p1 != p2)
					return p1 - p2;
				idx1++;
				idx2++;
			}
		}
	}

	//
	// Internal method of compare().
	//
	// It compares two cards with their extensions.
	//
	private int compareExpansion(Card c1, Card c2) {
		return c1.getExpansion().compareTo(c2.getExpansion());
	}

	/**
	 * Returns true if this object is equal to the specified one.
	 *
	 * It returns true, if both objects are equivalent.
	 *
	 * @param  o  an object to be compared
	 * @return  true if both objects are equivalent
	 */
	@Override
	public boolean equals(Object o) {
		if (o == this)
			return true;
		if (o == null || !(o instanceof CardComparator))
			return false;
		CardComparator comparator = (CardComparator)o;
		return (mode == comparator.mode);
	}

	/**
	 * Converts this object to a String.
	 *
	 * @return  a string "CLASS-NAME (MODE)"
	 */
	@Override
	public String toString() {
		return String.format("%s (%s)",
			getClass().getSimpleName(), mode.toString());
	}
}
