//
// Copyright (c) 2013  Motoyuki Kasahara
//
// This program is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 2 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program.  If not, see <http://www.gnu.org/licenses/>.
//
package jp.sourceforge.gamerandomizerlib;

/**
 * <p>
 * Exception class for GameBuilderXMLFile.
 * </p>
 *
 * @author   Motoyuki Kasahara
 * @version  1.0
 */
public final class GameBuilderXMLFileException extends GameBuilderException {
	private String fileTitle;
	private int lineNo;
	private String message;

	/**
	 * Constructs an object with the specified file title, message format
	 * and its arguments.
	 *
	 * @param  fileTitleArg  a file title
	 * @param  formatArg     a message format
	 * @param  args          arguments to the message format.
	 */
	public GameBuilderXMLFileException(String fileTitleArg, String formatArg,
		Object ... args) {
		super();

		fileTitle = fileTitleArg;
		lineNo    = 0;
		String header = String.format("%s: ", fileTitleArg);
		message = header + String.format(formatArg, args);
	}

	/**
	 * Constructs an object with the specified file title, line number,
	 * message format and its arguments.
	 *
	 * @param  fileTitleArg  a file title
	 * @param  lineNoArg     a line number
	 * @param  formatArg     a message format
	 * @param  args          arguments to the message format.
	 */
	public GameBuilderXMLFileException(String fileTitleArg, int lineNoArg,
		String formatArg, Object ... args) {
		super();

		fileTitle = fileTitleArg;
		lineNo    = lineNoArg;
		String header = String.format("%s: %d: ", fileTitleArg, lineNo);
		message = header + String.format(formatArg, args);
	}

	/**
	 * Constructs an object with the specified cause, file title, message
	 * format and its arguments.
	 *
	 * @param  causeArg      cause
	 * @param  fileTitleArg  a file title
	 * @param  formatArg     a message format
	 * @param  args          arguments to the message format.
	 */
	public GameBuilderXMLFileException(Throwable causeArg, 
		String fileTitleArg, String formatArg, Object ... args) {
		super(causeArg);

		fileTitle = fileTitleArg;
		lineNo    = 0;
		String header = String.format("%s: ", fileTitleArg);
		message = header + String.format(formatArg, args);
	}

	/**
	 * Constructs an object with the specified cause, file title, line number,
	 * message format and its arguments.
	 *
	 * @param  causeArg      cause
	 * @param  fileTitleArg  a file title
	 * @param  lineNoArg     a line number
	 * @param  formatArg     a message format
	 * @param  args          arguments to the message format.
	 */
	public GameBuilderXMLFileException(Throwable causeArg, 
		String fileTitleArg, int lineNoArg, String formatArg, 
		Object ... args) {
		super(causeArg);

		fileTitle = fileTitleArg;
		lineNo    = lineNoArg;
		String header = String.format("%s: %d: ", fileTitleArg, lineNo);
		message = header + String.format(formatArg, args);
	}

	/**
	 * Returns a file title (path to a file relative to the top directory).
	 *
	 * @return  a file title
	 */
	public String getFileTitle() {
		return fileTitle;
	}

	/**
	 * Returns a line number.
	 *
	 * @return  a line number.
	 */
	public int getLineNo() {
		return lineNo;
	}

	/**
	 * Returns an error message.
	 *
	 * @return  an error message.
	 */
	@Override
	public String getMessage() {
		return message;
	}

	/**
	 * Returns a localized error message.
	 *
	 * @return  a localized error message.
	 */
	@Override
	public String getLocalizedMessage() {
		return message;  // not supported yet.
	}
}
