//
// Copyright (c) 2013  Motoyuki Kasahara
//
// This program is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 2 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program.  If not, see <http://www.gnu.org/licenses/>.
//
package jp.sourceforge.gamerandomizerlib;

import java.util.List;
import java.util.Random;

import nedragtna.random.MTRandom;

/**
 * <p>
 * Selects cards randomly.
 * </p>
 *
 * <p>
 * Its object randomly selects cards registered in a 'Game' object.
 * </p>
 *
 * @author   Motoyuki Kasahara
 * @version  1.0
 */
public class Randomizer {
	private Game game;
	private Random random;

	/**
	 * Constructs an object.
	 *
	 * This object randomly selects the number of 'gameArg.selectionSize()'
	 * cards from the game object 'gameArg', using the random number
	 * generator of 'MTRandom' class, implementation of Mersenne Twister.
	 * An appropriate random seed is also set to the random number generator
	 * automatically.
	 *
	 * @param  gameArg           a game
	 */
	public Randomizer(Game gameArg) {
		game = new Game(gameArg);
		long seed = System.currentTimeMillis()
			+ Runtime.getRuntime().freeMemory();
		random = new MTRandom(seed);
	}

	/**
	 * Constructs an object.
	 *
	 * This object randomly selects the number of 'gameArg.selectionSize()'
	 * cards from the game object 'gameArg', using the random number
	 * generator 'randomArg'.
	 *
	 * Note that this object never sets a random seed of the random number
	 * generator.
	 *
	 * @param  gameArg           a game
	 * @param  randomArg         a random number generator
	 */
	public Randomizer(Game gameArg, Random randomArg) {
		assert randomArg != null;

		game = new Game(gameArg);
		random = randomArg;
	}

	/**
	 * Returns the 'Game' object bound to this object.
	 *
	 * @return  a 'Game' object.
	 */
	public Game getGame() {
		return new Game(game);
	}

	/**
	 * Returns a random number generator.
	 *
	 * @return  a random number generator
	 */
	public Random getRandom() {
		return random;
	}

	/**
	 * Select cards randomly.
	 *
	 * This methods randomly selects a card in a 'Game' object bound to this
	 * object and adds it to 'selectedCards'.  It repeats this operation
	 * until the size of 'selectedCards' reaches the limit (i.e. the
	 * selection size).
	 *
	 * It never chooses a card which has already been stored in
	 * 'selectedCards'.  If there is a card which must be chosen, add
	 * the card to 'selectedCards' before call this method.
	 *
	 * When there is no more card to be chosen in 'game', it returns
	 * immediately.  In this case, the size of 'selectedCards' is less than
	 * the limit.
	 *
	 * If the size of given 'selectedCards' exceeds the limit, this method
	 * randomly removes cards in it, instead.
	 *
	 * @param  selectedCards  a list in which selected cards are stored
	 * @return  the number of selected cards, including cards initially
	 *          stored in 'selectedCards'.
	 */
	public int select(List<Card> selectedCards) {
		assert selectedCards != null;

		int nSelected = selectedCards.size();
		int selectionSize = game.getSelectionSize();

		if (nSelected < selectionSize) {
			Game g = new Game(game);
			g.removeCards(selectedCards);
			int nCandidates = g.getCardsSize();
			while (nSelected < selectionSize && nCandidates > 0) {
				int index = random.nextInt(nCandidates);
				Card c = g.getCard(index);
				selectedCards.add(c);
				g.removeCard(c);
				nSelected++;
				nCandidates--;
			}
		} else {
			while (nSelected > selectionSize) {
				int index = random.nextInt(nSelected);
				selectedCards.remove(index);
				nSelected--;
			}
		}

		return nSelected;
	}

	/**
	 * Converts this object to a String.
	 *
	 * @return  a string "CLASS-NAME for GAME"
	 */
	@Override
	public String toString() {
		return String.format("%s for %s",
			getClass().getSimpleName(), game.toString());
	}
}
