//
// Copyright (c) 2013  Motoyuki Kasahara
//
// This program is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 2 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program.  If not, see <http://www.gnu.org/licenses/>.
//
package jp.sourceforge.gamerandomizerlib;

import java.util.ArrayList;
import java.util.List;
import java.util.Random;
import junit.framework.*;
import junit.textui.TestRunner;
import nedragtna.random.MTRandom;

public final class RandomizerTest extends TestCase {
	public RandomizerTest(String name) {
		super(name);
	}

	//
	// Create a 'Game' object.
	//
	private Game buildGame(int selectionSize, int cardsSize) {
		Game g = new Game("test-game-id", "", selectionSize);
		Expansion x = new Expansion("test-expansion-id", "");
		g.addExpansion(x);
		for (int i = 0; i < cardsSize; i++)
			g.addCard(new Card("test-card-id" + i, "", "", x));
		return g;
	}

	//
	// Test Randomizer() and getRandom().
	//
	public void testCostructor1() {
		try {
			Game g = buildGame(3, 6);
			Randomizer randomizer = new Randomizer(g);
			assertTrue(randomizer.getRandom() instanceof MTRandom);
		} catch (Exception e) {
			fail(e.getMessage());
		}
	}

	//
	// Test Randomizer(game, random) and getRandom().
	//
	public void testCostructor2() {
		try {
			Game g = buildGame(3, 6);
			Random r = new Random();
			Randomizer randomizer = new Randomizer(g, r);
			assertTrue(randomizer.getRandom() instanceof Random);
		} catch (Exception e) {
			fail(e.getMessage());
		}
	}

	//
	// Test getGame().
	//
	public void getGame() {
		try {
			Game g1 = buildGame(3, 6);
			Randomizer randomizer = new Randomizer(g1);

			Game g2 = randomizer.getGame();
			assertTrue(g1.equals(g2));

			List<Expansion> xl1 = g1.getExpansions();
			List<Expansion> xl2 = g2.getExpansions();
			assertTrue(xl1.size() == xl2.size());
			for (int i = 0; i < xl1.size(); i++)
				assertTrue(xl1.get(i).equals(xl2.get(i)));

			List<Card> cl1 = g1.getCards();
			List<Card> cl2 = g2.getCards();
			assertTrue(cl1.size() == cl2.size());
			for (int i = 0; i < cl1.size(); i++)
				assertTrue(cl1.get(i).equals(cl2.get(i)));
		} catch (Exception e) {
			fail(e.getMessage());
		}
	}

	//
	// Validate a list of cards selected by 'Randomizer'.
	//
	private boolean validateSelectedCards(List<Card> selectedCards,
		List<Card> cards) {
		int selectedSize = selectedCards.size();

		//
		// Check if all cards in 'selectedCards' are also in 'cards'.
		//
		for (int i = 0; i < selectedSize; i++) {
			Card c = selectedCards.get(i);
			if (cards.indexOf(c) < 0)
				return false;
		}

		//
		// Check if all cards in 'selectedCards' are unique.
		//
		for (int i = 0; i < selectedSize - 1; i++) {
			Card c = selectedCards.get(i);
			for (int j = i + 1; j < selectedSize; j++) {
				if (c.equals(selectedCards.get(j)))
					return false;
			}
		}

		return true;
	}

	//
	// Test select(selectedCards) -- a simple case.
	//
	public void testSelect1() {
		try {
			int selectionSize = 3;
			int cardsSize = 6;
			Game g = buildGame(selectionSize, cardsSize);

			Randomizer randomizer = new Randomizer(g);
			List<Card> cl = new ArrayList<Card>();
			int selectResult = randomizer.select(cl);

			assertEquals(selectResult, selectionSize);
			assertEquals(cl.size(), selectionSize);
			validateSelectedCards(cl, g.getCards());
		} catch (Exception e) {
			fail(e.getMessage());
		}
	}

	//
	// Test select(selectedCards) -- 'selectedCards' is not an empty.
	//
	public void testSelect2() {
		try {
			int selectionSize = 4;
			int cardsSize = 8;
			Game g = buildGame(selectionSize, cardsSize);

			Randomizer randomizer = new Randomizer(g);
			List<Card> cl = new ArrayList<Card>();
			cl.add(g.getCard(0));
			cl.add(g.getCard(1));
			int selectResult = randomizer.select(cl);

			assertEquals(selectResult, selectionSize);
			assertEquals(cl.size(), selectionSize);
			assertEquals(cl.get(0), g.getCard(0));
			assertEquals(cl.get(1), g.getCard(1));
			validateSelectedCards(cl, g.getCards());
		} catch (Exception e) {
			fail(e.getMessage());
		}
	}

	//
	// Test select(selectedCards) -- 'selectedCards' is too long.
	// 
	//
	public void testSelect3() {
		try {
			int selectionSize = 3;
			int cardsSize = 6;
			Game g = buildGame(selectionSize, cardsSize);

			Randomizer randomizer = new Randomizer(g);
			List<Card> cl = new ArrayList<Card>(g.getCards());
			int selectResult = randomizer.select(cl);

			assertEquals(selectResult, selectionSize);
			assertEquals(cl.size(), selectionSize);
			validateSelectedCards(cl, g.getCards());
		} catch (Exception e) {
			fail(e.getMessage());
		}
	}

	//
	// Test select(selectedCards) -- not enough cards.
	//
	public void testSelect4() {
		try {
			int selectionSize = 3;
			int cardsSize = 2;
			Game g = buildGame(selectionSize, cardsSize);

			Randomizer randomizer = new Randomizer(g);
			List<Card> cl = new ArrayList<Card>();
			int selectResult = randomizer.select(cl);

			assertEquals(selectResult, cardsSize);
			assertEquals(cl.size(), cardsSize);
			validateSelectedCards(cl, g.getCards());
		} catch (Exception e) {
			fail(e.getMessage());
		}
	}

	//
	// Test toString().
	//
	public void testToString() {
		try {
			Game g = buildGame(3, 6);
			Randomizer randomizer = new Randomizer(g);
			String endString = "Randomizer for " + g.getTitle();
			assertTrue(randomizer.toString().endsWith(endString));
		} catch (Exception e) {
			fail(e.getMessage());
		}
	}

	public static void main(String[] args) {
		junit.textui.TestRunner.run(RandomizerTest.class);
	}
}
